<?php

namespace Assetdocs\Stock\Http\Controllers;

use App\Http\Controllers\Controller as BaseController;
use Form;
use Assetdocs\Stock\Http\Requests\StockAdminWebRequest;
use Assetdocs\Stock\Interfaces\StockRepositoryInterface;
use Assetdocs\Stock\Models\Stock;

/**
 * Admin web controller class.
 */
class StockAdminController extends BaseController
{
    /**
     * The authentication guard that should be used.
     *
     * @var string
     */
    protected $guard = 'admin.web';

    /**
     * Initialize stock controller.
     *
     * @param type StockRepositoryInterface $stock
     *
     * @return type
     */
    public function __construct(StockRepositoryInterface $stock)
    {
        $this->middleware('web');
        $this->middleware('auth:admin.web');
        $this->setupTheme(config('theme.themes.admin.theme'), config('theme.themes.admin.layout'));

        $this->repository = $stock;
        parent::__construct();
    }

    /**
     * Display a list of stock.
     *
     * @return Response
     */
    public function index(StockAdminWebRequest $request)
    {
        $pageLimit = $request->input('pageLimit');
        if ($request->wantsJson()) {
            $stocks  = $this->repository
            ->pushCriteria(new \Assetdocs\Stock\Repositories\Criteria\StockAdminCriteria())
                ->setPresenter('\\Assetdocs\\Stock\\Repositories\\Presenter\\StockListPresenter')
                ->scopeQuery(function ($query) {
                    return $query->orderBy('id', 'DESC');
                })->paginate($pageLimit);

            $stocks['recordsTotal']    = $stocks['meta']['pagination']['total'];
            $stocks['recordsFiltered'] = $stocks['meta']['pagination']['total'];
            $stocks['request']         = $request->all();
            return response()->json($stocks, 200);

        }

        $this   ->theme->prependTitle(trans('stock::stock.names').' :: ');
        return $this->theme->of('stock::admin.stock.index')->render();
    }

    /**
     * Display stock.
     *
     * @param Request $request
     * @param Model   $stock
     *
     * @return Response
     */
    public function show(StockAdminWebRequest $request, Stock $stock)
    {
        if (!$stock->exists) {
            return response()->view('stock::admin.stock.new', compact('stock'));
        }

        Form::populate($stock);
        return response()->view('stock::admin.stock.show', compact('stock'));
    }

    /**
     * Show the form for creating a new stock.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(StockAdminWebRequest $request)
    {

        $stock = $this->repository->newInstance([]);

        Form::populate($stock);

        return response()->view('stock::admin.stock.create', compact('stock'));

    }

    /**
     * Create new stock.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(StockAdminWebRequest $request)
    {
        try {
            $attributes             = $request->all();
            $attributes['user_id']  = user_id('admin.web');
            $stock          = $this->repository->create($attributes);

            return response()->json([
                'message'  => trans('messages.success.updated', ['Module' => trans('stock::stock.name')]),
                'code'     => 204,
                'redirect' => trans_url('/admin/stock/stock/'.$stock->getRouteKey())
            ], 201);


        } catch (Exception $e) {
            return response()->json([
                'message'  => $e->getMessage(),
                'code'     => 400,
            ], 400);
        }
    }

    /**
     * Show stock for editing.
     *
     * @param Request $request
     * @param Model   $stock
     *
     * @return Response
     */
    public function edit(StockAdminWebRequest $request, Stock $stock)
    {
        Form::populate($stock);
        return  response()->view('stock::admin.stock.edit', compact('stock'));
    }

    /**
     * Update the stock.
     *
     * @param Request $request
     * @param Model   $stock
     *
     * @return Response
     */
    public function update(StockAdminWebRequest $request, Stock $stock)
    {
        try {

            $attributes = $request->all();

            $stock->update($attributes);

            return response()->json([
                'message'  => trans('messages.success.updated', ['Module' => trans('stock::stock.name')]),
                'code'     => 204,
                'redirect' => trans_url('/admin/stock/stock/'.$stock->getRouteKey())
            ], 201);

        } catch (Exception $e) {

            return response()->json([
                'message'  => $e->getMessage(),
                'code'     => 400,
                'redirect' => trans_url('/admin/stock/stock/'.$stock->getRouteKey()),
            ], 400);

        }
    }

    /**
     * Remove the stock.
     *
     * @param Model   $stock
     *
     * @return Response
     */
    public function destroy(StockAdminWebRequest $request, Stock $stock)
    {

        try {

            $t = $stock->delete();

            return response()->json([
                'message'  => trans('messages.success.deleted', ['Module' => trans('stock::stock.name')]),
                'code'     => 202,
                'redirect' => trans_url('/admin/stock/stock/0'),
            ], 202);

        } catch (Exception $e) {

            return response()->json([
                'message'  => $e->getMessage(),
                'code'     => 400,
                'redirect' => trans_url('/admin/stock/stock/'.$stock->getRouteKey()),
            ], 400);
        }
    }
}
