<?php

namespace Assetdocs\Stock\Http\Controllers;

use App\Http\Controllers\Controller as BaseController;
use Assetdocs\Stock\Http\Requests\StockUserApiRequest;
use Assetdocs\Stock\Interfaces\StockRepositoryInterface;
use Assetdocs\Stock\Models\Stock;

/**
 * User API controller class.
 */
class StockUserApiController extends BaseController
{

    /**
     * The authentication guard that should be used.
     *
     * @var string
     */
    protected $guard = 'api';

    /**
     * Initialize stock controller.
     *
     * @param type StockRepositoryInterface $stock
     *
     * @return type
     */
    public function __construct(StockRepositoryInterface $stock)
    {
        $this->middleware('api');
        $this->middleware('jwt.auth:api');
        $this->setupTheme(config('theme.themes.user.theme'), config('theme.themes.user.layout'));

        $this->repository = $stock;
        parent::__construct();
    }

    /**
     * Display a list of stock.
     *
     * @return json
     */
    public function index(StockUserApiRequest $request)
    {
        $stocks  = $this->repository
            ->pushCriteria(new \Lavalite\Stock\Repositories\Criteria\StockUserCriteria())
            ->setPresenter('\\Assetdocs\\Stock\\Repositories\\Presenter\\StockListPresenter')
            ->scopeQuery(function($query){
                return $query->orderBy('id','DESC');
            })->all();
        $stocks['code'] = 2000;
        return response()->json($stocks) 
            ->setStatusCode(200, 'INDEX_SUCCESS');

    }

    /**
     * Display stock.
     *
     * @param Request $request
     * @param Model   Stock
     *
     * @return Json
     */
    public function show(StockUserApiRequest $request, Stock $stock)
    {

        if ($stock->exists) {
            $stock         = $stock->presenter();
            $stock['code'] = 2001;
            return response()->json($stock)
                ->setStatusCode(200, 'SHOW_SUCCESS');;
        } else {
            return response()->json([])
                ->setStatusCode(400, 'SHOW_ERROR');
        }

    }

    /**
     * Show the form for creating a new stock.
     *
     * @param Request $request
     *
     * @return json
     */
    public function create(StockUserApiRequest $request, Stock $stock)
    {
        $stock         = $stock->presenter();
        $stock['code'] = 2002;
        return response()->json($stock)
            ->setStatusCode(200, 'CREATE_SUCCESS');
    }

    /**
     * Create new stock.
     *
     * @param Request $request
     *
     * @return json
     */
    public function store(StockUserApiRequest $request)
    {
        try {
            $attributes             = $request->all();
            $attributes['user_id']  = user_id('admin.api');
            $stock          = $this->repository->create($attributes);
            $stock          = $stock->presenter();
            $stock['code']  = 2004;

            return response()->json($stock)
                ->setStatusCode(201, 'STORE_SUCCESS');
        } catch (Exception $e) {
            return response()->json([
                'message' => $e->getMessage(),
                'code'    => 4004,
            ])->setStatusCode(400, 'STORE_ERROR');
        }

    }

    /**
     * Show stock for editing.
     *
     * @param Request $request
     * @param Model   $stock
     *
     * @return json
     */
    public function edit(StockUserApiRequest $request, Stock $stock)
    {
        if ($stock->exists) {
            $stock         = $stock->presenter();
            $stock['code'] = 2003;
            return response()-> json($stock)
                ->setStatusCode(200, 'EDIT_SUCCESS');;
        } else {
            return response()->json([])
                ->setStatusCode(400, 'SHOW_ERROR');
        }
    }

    /**
     * Update the stock.
     *
     * @param Request $request
     * @param Model   $stock
     *
     * @return json
     */
    public function update(StockUserApiRequest $request, Stock $stock)
    {
        try {

            $attributes = $request->all();

            $stock->update($attributes);
            $stock         = $stock->presenter();
            $stock['code'] = 2005;

            return response()->json($stock)
                ->setStatusCode(201, 'UPDATE_SUCCESS');


        } catch (Exception $e) {

            return response()->json([
                'message'  => $e->getMessage(),
                'code'     => 4005,
            ])->setStatusCode(400, 'UPDATE_ERROR');

        }
    }

    /**
     * Remove the stock.
     *
     * @param Request $request
     * @param Model   $stock
     *
     * @return json
     */
    public function destroy(StockUserApiRequest $request, Stock $stock)
    {

        try {

            $t = $stock->delete();

            return response()->json([
                'message'  => trans('messages.success.delete', ['Module' => trans('stock::stock.name')]),
                'code'     => 2006
            ])->setStatusCode(202, 'DESTROY_SUCCESS');

        } catch (Exception $e) {

            return response()->json([
                'message'  => $e->getMessage(),
                'code'     => 4006,
            ])->setStatusCode(400, 'DESTROY_ERROR');
        }
    }
}
