<?php

namespace Assetdocs\Stock\Http\Controllers;

use App\Http\Controllers\Controller as BaseController;
use Form;
use Assetdocs\Stock\Http\Requests\StockUserWebRequest;
use Assetdocs\Stock\Interfaces\StockRepositoryInterface;
use Assetdocs\Stock\Models\Stock;

class StockUserController extends BaseController
{
    /**
     * The authentication guard that should be used.
     *
     * @var string
     */
    protected $guard = 'web';

    /**
     * Initialize stock controller.
     *
     * @param type StockRepositoryInterface $stock
     *
     * @return type
     */
    public function __construct(StockRepositoryInterface $stock)
    {
        $this->middleware('web');
        $this->middleware('auth:web');
        $this->setupTheme(config('theme.themes.user.theme'), config('theme.themes.user.layout'));

        $this->repository = $stock;
        parent::__construct();
    }

    /**
     * Display a listing of the resource.
     *
     * @return Response
     */
    public function index(StockUserWebRequest $request)
    {
        $this->repository->pushCriteria(new \Assetdocs\Stock\Repositories\Criteria\StockUserCriteria());
        $stocks = $this->repository->scopeQuery(function($query){
            return $query->orderBy('id','DESC');
        })->paginate();

        $this->theme->prependTitle(trans('stock::stock.names').' :: ');

        return $this->theme->of('stock::user.stock.index', compact('stocks'))->render();
    }

    /**
     * Display the specified resource.
     *
     * @param Request $request
     * @param Stock $stock
     *
     * @return Response
     */
    public function show(StockUserWebRequest $request, Stock $stock)
    {
        Form::populate($stock);

        return $this->theme->of('stock::user.stock.show', compact('stock'))->render();
    }

    /**
     * Show the form for creating a new resource.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(StockUserWebRequest $request)
    {

        $stock = $this->repository->newInstance([]);
        Form::populate($stock);

        return $this->theme->of('stock::user.stock.create', compact('stock'))->render();
    }

    /**
     * Display the specified resource.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(StockUserWebRequest $request)
    {
        try {
            $attributes = $request->all();
            $attributes['user_id'] = user_id();
            $stock = $this->repository->create($attributes);

            return redirect(trans_url('/user/stock/stock'))
                -> with('message', trans('messages.success.created', ['Module' => trans('stock::stock.name')]))
                -> with('code', 201);

        } catch (Exception $e) {
            redirect()->back()->withInput()->with('message', $e->getMessage())->with('code', 400);
        }
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param Request $request
     * @param Stock $stock
     *
     * @return Response
     */
    public function edit(StockUserWebRequest $request, Stock $stock)
    {

        Form::populate($stock);

        return $this->theme->of('stock::user.stock.edit', compact('stock'))->render();
    }

    /**
     * Update the specified resource.
     *
     * @param Request $request
     * @param Stock $stock
     *
     * @return Response
     */
    public function update(StockUserWebRequest $request, Stock $stock)
    {
        try {
            $this->repository->update($request->all(), $stock->getRouteKey());

            return redirect(trans_url('/user/stock/stock'))
                ->with('message', trans('messages.success.updated', ['Module' => trans('stock::stock.name')]))
                ->with('code', 204);

        } catch (Exception $e) {
            redirect()->back()->withInput()->with('message', $e->getMessage())->with('code', 400);
        }
    }

    /**
     * Remove the specified resource.
     *
     * @param int $id
     *
     * @return Response
     */
    public function destroy(StockUserWebRequest $request, Stock $stock)
    {
        try {
            $this->repository->delete($stock->getRouteKey());
            return response()->json([
                'message'  => trans('messages.success.deleted', ['Module' => trans('stock::stock.name')]),
                'code'     => 202,
                'redirect' => trans_url('/user/stock/stock/0'),
            ], 202);

        } catch (Exception $e) {

            redirect()->back()->withInput()->with('message', $e->getMessage())->with('code', 400);

        }
    }
}
